// Copyright 2009 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

// +build ignore

// Generate a 64bit and a 63bit version of the vector of numbers which
// is used for seeding the ALFG in /math/rand/rng.go (rng_cooked).

package main

import "fmt"

const (
	LEN  = 607
	TAP  = 273
	MASK = (1 << 63) - 1
	A    = 48271
	M    = 2147483647
	Q    = 44488
	R    = 3399
)

var (
	rng_vec           [LEN]int64
	rng_tap, rng_feed int
)

func seedrand(x int32) int32 {
	hi := x / Q
	lo := x % Q
	x = A*lo - R*hi
	if x < 0 {
		x += M
	}
	return x
}

func srand(seed int32) {
	rng_tap = 0
	rng_feed = LEN - TAP
	seed = seed % M
	if seed < 0 {
		seed += M
	}
	if seed == 0 {
		seed = 89482311
	}
	x := seed
	for i := -20; i < LEN; i++ {
		x = seedrand(x)
		if i >= 0 {
			var u int64
			u = int64(x) << 20
			x = seedrand(x)
			u ^= int64(x) << 10
			x = seedrand(x)
			u ^= int64(x)
			rng_vec[i] = u
		}
	}
}

func vrand() int64 {
	rng_tap--
	if rng_tap < 0 {
		rng_tap += LEN
	}
	rng_feed--
	if rng_feed < 0 {
		rng_feed += LEN
	}
	x := (rng_vec[rng_feed] + rng_vec[rng_tap])
	rng_vec[rng_feed] = x
	return x
}

func main() {
	srand(1)
	for i := 0; i < 780e10; i++ {
		vrand()
	}
	fmt.Println("rng_vec after 780e10 calls to vrand:")
	fmt.Printf("%#v\n", rng_vec)
	for i := range rng_vec {
		rng_vec[i] = rng_vec[i] & MASK
	}
	fmt.Println("lower 63bit of rng_vec after 780e10 calls to vrand:")
	fmt.Printf("%#v\n", rng_vec)
}
